#!/usr/bin/env python
#    Created by Eugenio Cutolo  <me@eugesoftware.com>
#
#    This program can be distributed under the terms of the GNU GPL.
#    See the file COPYING.
#

#Pre-relase for tracker QtFrontend

import sys,os,dbus
from mainform import *

class TrackerClient:

	def initclient(self):
		bus = dbus.SessionBus()
		obj = bus.get_object('org.freedesktop.Tracker','/org/freedesktop/tracker')
		#self.tracker = dbus.Interface(obj, 'org.freedesktop.Tracker')
		self.keywords_iface = dbus.Interface(obj, 'org.freedesktop.Tracker.Keywords')
		self.search_iface = dbus.Interface(obj, 'org.freedesktop.Tracker.Search')
		self.files_iface = dbus.Interface(obj, 'org.freedesktop.Tracker.Files')
		
		#self.version = self.tracker.GetVersion()

		#Self service eheheh
		#self.services = self.tracker.GetServices(True)
		self.services = ['Files','Development Files','Documents','Images','Music','Text Files','Videos']
		self.query_id = 0

	def search(self,mode,service,input):
		self.fatalerr = 0
		if mode == 0:
			self.filelist = self.search_iface.Text(	1,service,input,0,-1)
			self.filelist = map(lambda x: str(x),self.filelist)
		elif mode == 1: 
			self.filelist = self.keywords_iface.Search(-1,service,[input,] ,-1,-1)
			self.filelist = map(lambda x: str(x),self.filelist)
			
		if len(self.filelist) == 0:
			self.on_tracker_error("Nothing found with "+input)

#Thank you Mikkel
	def text_snippet(self, text, result, service='Files'):
		snippet = self.search_iface.GetSnippet(service, result, text)
		snippet = snippet.replace('<!--', '&lt;!--').strip()
		return snippet

	def on_tracker_error(self, e):
		print "Error:"+e
		self.fatalerr = 1

class TrackerGui(MainForm,TrackerClient):
	
	def _refresh_page(self):
		self.result_list.clear()
		
		if self.pagen > 0:
			self.prevbtn.setEnabled(1)
		elif self.pagen == 0:
			self.prevbtn.setEnabled(0)
		if len(self.filelist) > 5 and self.pagen < len(self.filelist) / 5:
			self.nextbtn.setEnabled(1)
		else:
			self.nextbtn.setEnabled(0)

		self.pagen_display.setText("Results "+str((5*self.pagen)+1)+" - "+str((5*self.pagen)+5)+" of about "+str(len(self.filelist)))
		results = range(5*self.pagen,(5*self.pagen)+5)
		self._show_result(results)

	def _show_result(self,result):
		for f in result:
			if f < len(self.filelist):
				file = self.filelist[f]
				service = self.files_iface.GetServiceType(file)
				item = QListViewItem(self.result_list,None)
				item.setMultiLinesEnabled(1)
				if len(os.path.dirname(file)) > 40:
					dirname = os.path.dirname(file).replace(os.path.dirname(file)[0:40],os.path.dirname(file)[0:40]+"\n")
				else:
					dirname = os.path.dirname(file)
				item.setText(0,os.path.basename(file)+"\n"+dirname+"\n"+service)
				self.mime_icon = QPixmap(self._get_iconc(service))
				item.setPixmap(0,self.mime_icon)
				item.setText(1,self.text_snippet(self.input,file))

#For now I'm using external icon(not kde current theme)
	def _get_iconc(self,service):
		return "./"+service.replace(" Files","").lower()+".png"
	
	def on_tracker_error(self, e):
		self.result_list.clear()
		self.pagen_display.setText(e)
		self.fatalerr = 1

#For now the file will be opened with konqueror
	def konq_open(self):
		info = str(self.result_list.currentItem().text(0)).split("\n")
		os.spawnlp(os.P_WAIT, 'konqueror', 'konqueror', str(info[1].strip('\n')+'/'+info[0]))
		
#----------------Qt Events----------------------------
	
	def findbtn_clicked(self):
		self.input = str(self.searchinp.text())
		service = self.services[self.services_combo.currentItem()]
		self.search(self.mode_combo.currentItem(),service,self.input)
		self.prevbtn.setEnabled(0)
		self.nextbtn.setEnabled(0)
		self.pagen = 0
		if self.fatalerr == 0: self._refresh_page()

	def nextbtn_clicked(self):
		self.result_list.clear()
		self.pagen = self.pagen + 1
		self._refresh_page()

	def prevbtn_clicked(self):
		self.result_list.clear()
		self.pagen = self.pagen - 1
		self._refresh_page()

	def result_list_doubleClicked(self,item):
		self.konq_open()

	def result_list_contextMenuRequested(self,item):
		print item
		if item != self.result_list:
			info = str(item.text(0)).split("\n")
			contextMenu = QPopupMenu(self)
			contextMenu.insertItem( "&Open with Konqueror", self.konq_open, Qt.CTRL+Qt.Key_O )
			contextMenu.insertItem( "&Open with...", self.konq_open, Qt.CTRL+Qt.Key_S )
			contextMenu.exec_loop( QCursor.pos() )

if __name__ == "__main__":
	app = QApplication(sys.argv)
	gui = TrackerGui()
	gui.initclient()
	gui.show()
	app.setMainWidget(gui)
	app.exec_loop()